;
; ------------------------------------------------------------
;
;   PureBasic - Print/Preview Module
;
;   FileName CDPrint.pbi
;
;   
; ------------------------------------------------------------
;
;UseJPEGImageDecoder()
DeclareModule CDPrint
  
  ;Page Variables This will change as I proceed to maybe an array of a page structure
  Global  PageHeight.i ;Height of Selected Page in mm
  Global PageWidth.i  ;Width of Selected Page in mm
  Global TopMargin.i  ;Top Margin In mm
  Global LeftMargin.i ;Left Margin In mm
  Global  BottomMargin.i ;Bottom Margin In mm
  Global  RightMargin.i  ;Right Margin In mm
  
  Global OkPressed.i = #False
  
 ;Experimenting with Orientation to attempt to allow a mix of portrait and Landscape in the same document 
  Global  Orientation.i = -1 ;Portrait or Landscape
  
  Global  Preview.i
  Global PreviewWindow.l
  GPageHeight.i ;mm To work out Graphic scale factor
  GPageWidth.i  ;mm To work out Graphic scale factor
  Orientation.i ;Portrait or Landscape for print routines
  TPageHeight.i ;Points To work out Text scale factor
  TPageWidth.i  ;Points To work out Text scale factor
  
  Declare CDPrintEvents(Event)
  Declare.l open(JobName.s,PageHeight,PageWidth)
  Declare ShowPage(PageID)
  Declare AddPage(PageHeight,PageWidth)
  Declare Finished()
  Declare PrintLine(Startx,Starty,Endx,Endy,LineWidth)
  Declare PrintBox(Topx,Topy,Bottomx,Bottomy,LineWidth)
  Declare PrintText(Startx,Starty,Font.s,Size.i,Text.s)
  Declare PrintImage(Image.l,Topx.i,Topy.i,Width.i,Height.i)
  Declare.f GettextWidthmm(text.s,FName.s,FSize.f)
  Declare.f GettextHeightmm(text.s,FName.s,FSize.f)
  Declare Page_Setup()
  
EndDeclareModule

Module CDPrint
  
  ;Structure For Usefull Printer Information
  Structure PInfo
    TopMargin.i
    LeftMargin.i
    BottomMargin.i
    RightMargin.i
    Width.i
    Height.i
    UsableWidth.i
    UsableHeight.i
  EndStructure

  Global PrinterInfo.PInfo

  Global Dim PageNos.i(1)
  Global PageNo.i
  Global GraphicScale.f
  Global TextScale.f  
  Global Result
  
  ;PageSetup Form Variables
  Global frm_Printer, btn_Cancel, btn_Ok, cmb_Printers, cmb_PaperSize, frm_Margins, str_Top, str_Left, String_0, str_Bottom, str_Right, String_0_Copy3, img_Back, img_Page, opt_Portrait, opt_Landscape, frm_Orientation
  
  Procedure GetPrinterInfo()
  
  CompilerSelect #PB_Compiler_OS
    
    CompilerCase   #PB_OS_MacOS
      
      ;The vectordrawing functions print correctly on the MAC so simply set all to zero
      PrinterInfo\Width = 0
      PrinterInfo\Height = 0
      PrinterInfo\UsableWidth = 0
      PrinterInfo\UsableHeight = 0
      PrinterInfo\TopMargin = 0
      PrinterInfo\LeftMargin = 0
      PrinterInfo\BottomMargin = 0
      PrinterInfo\RightMargin = 0

    CompilerCase   #PB_OS_Linux   
      
      ;Not Defined Yet
      
    CompilerCase   #PB_OS_Windows   
      
      Define HDPmm.d
      Define VDPmm.d
        
      printer_DC.l = StartDrawing(PrinterOutput())

      If printer_DC
        HDPmm = GetDeviceCaps_(printer_DC,#LOGPIXELSX) / 25.4
        VDPmm = GetDeviceCaps_(printer_DC,#LOGPIXELSY) / 25.4
        PrinterInfo\Width = GetDeviceCaps_(printer_DC,#PHYSICALWIDTH) / HDPmm
        PrinterInfo\Height = GetDeviceCaps_(printer_DC,#PHYSICALHEIGHT) / VDPmm
        PrinterInfo\UsableWidth = GetDeviceCaps_(printer_DC,#HORZSIZE) ; Returns mm
        PrinterInfo\UsableHeight = GetDeviceCaps_(printer_DC,#VERTSIZE); Returns mm
        PrinterInfo\TopMargin = GetDeviceCaps_(printer_DC,#PHYSICALOFFSETY) / VDPmm
        PrinterInfo\LeftMargin = GetDeviceCaps_(printer_DC,#PHYSICALOFFSETX) / HDPmm
        PrinterInfo\BottomMargin = PrinterInfo\Height - PrinterInfo\UsableHeight - PrinterInfo\TopMargin
        PrinterInfo\RightMargin = PrinterInfo\Width - PrinterInfo\UsableWidth - PrinterInfo\LeftMargin
      EndIf

      StopDrawing()
      
  CompilerEndSelect
   
EndProcedure
  
  Procedure.f GettextWidthmm(text.s,FName.s,FSize.f)
    
    LoadFont(0,FName.s, FSize.f)    ;Load Font In Points
    ASize.f = FSize.f * 0.352777778 ;Convert Font Points To mm
    VectorFont(FontID(0), ASize.f ) ;Use Font In mm Size
    ProcedureReturn VectorTextWidth(text.s,#PB_VectorText_Visible) ;Width of text In mm

  EndProcedure
  
  Procedure.f GettextHeightmm(text.s,FName.s,FSize.f)
    
    LoadFont(0,FName.s, FSize.f)    ;Load Font In Points
    ASize.f = FSize.f * 0.352777778 ;Convert Font Points To mm
    VectorFont(FontID(0), ASize.f)  ;Use Font In mm Size
    ProcedureReturn VectorTextHeight(text.s,#PB_VectorText_Visible) ;Height of text In mm

  EndProcedure 
  
  Procedure ShowPage(PageID)
    
    If CDPrint::Preview.i = 1
      SetGadgetState(66,PageID + 1)
      SetGadgetState(34, ImageID(PageNos.i(PageID)))
    
      ;Centre image gadget in window area
      If GadgetHeight(34) > GadgetWidth(34)

        Left  = (500 - GadgetWidth(34)) /2

        Top = 25

      Else

        Left = 0
         
        Top =  ((500-GadgetHeight(34)) /2 ) + 25
        
      EndIf
      ResizeGadget(34,Left,Top,#PB_Ignore,#PB_Ignore)
    EndIf  
        
  EndProcedure
  
  Procedure.l Open(JobName.s,PageHeight,PageWidth)

    TPageHeight.i = PageHeight * 2.834645669 ;mm To Points
    TPageWidth.i = PageWidth * 2.834645669

    If PageHeight > GPageWidth.i
      GraphicScale.f = 500/PageHeight
      TextScale.f = 500/TPageHeight.i
    Else
      GraphicScale.f = 500/PageWidth
      TextScale.f = 500/TPagewidth.i
    EndIf
  
    If Preview.i = 1 ;If preview mode open Preview Window
    
      PreviewWindow.l = OpenWindow(#PB_Any, #PB_Ignore,#PB_Ignore, 500, 535, "Print Preview - " + JobName.s)
      
      If PreviewWindow.l > 0
        
        SpinGadget     (66, 450, 0, 50, 25, 0, 1000,#PB_Spin_Numeric)
        SetGadgetState (66, 1)
        ImageGadget(34, 5, 5, 50, 50,  0,#PB_Image_Raised)

        ;Set Page Counter To Zero And Create first Page Image
        PageNo.i = 0 
        
        PageNos.i(PageNo.i) = CreateImage(#PB_Any, PageWidth * GraphicScale.f,PageHeight * GraphicScale.f, 32,RGB(255,255,255))
        ShowPage(PageNo.i)
        
        StartVectorDrawing(ImageVectorOutput(PageNos.i(PageNo.i) ))
        
        ;Set all printer margins to zero for preview
        PrinterInfo\TopMargin = 0
        PrinterInfo\LeftMargin = 0
        PrinterInfo\BottomMargin = 0
        PrinterInfo\RightMargin = 0
        ProcedureReturn PreviewWindow.l
        
      Else
      
        ;Tell The User We Failed
      
        ProcedureReturn 0
 
      EndIf
    
    Else
    
      ;Start Printing
      If PrintRequester()
        GetPrinterInfo()
        result = StartPrinting(JobName.s)
        ;Check Printer Paper Size and Orientation

        StartVectorDrawing(PrinterVectorOutput(#PB_Unit_Millimeter)) 

        If Orientation = 0
          RotateCoordinates(0,0,-90)
          TranslateCoordinates(-297, 0)
        EndIf
        ProcedureReturn result
      Else
        ProcedureReturn 0
      EndIf
    EndIf
  
  EndProcedure
  
  Procedure AddPage(PageHeight,PageWidth)
    
    If Preview.i = 1
      
      ;Increase Last Page Number  
      PageNo.i = PageNo.i + 1
      
      ;Increase Handle Array by one
      ReDim PageNos.i(PageNo.i)

      ;Show new page number in spin gadget
      
      SetGadgetState(66,PageNo.i + 1)
      
      SetGadgetText(66, Str(PageNo.i + 1))

      ;Create the image for the page and store handle in array
      PageNos.i(PageNo.i) = CreateImage(#PB_Any, PageWidth * GraphicScale.f,PageHeight * GraphicScale.f, 32,RGB(255,255,255))

      StartVectorDrawing(ImageVectorOutput(PageNos.i(PageNo.i) ))
       
    Else
      
      ;Printer New Page
      NewPrinterPage()
      If Orientation = 0
        RotateCoordinates(0,0,-90)
        TranslateCoordinates(-297, 0)
      EndIf
    EndIf
    
  EndProcedure
  
  Procedure PrintLine(Startx,Starty,Endx,Endy,LineWidth)
    
    If Preview.i = 1
      
      ;Scaled To Fit Preview Window   
      MovePathCursor(Startx * GraphicScale.f, Starty * GraphicScale.f)
        
      AddPathLine(Endx * GraphicScale.f, Endy * GraphicScale.f, #PB_Path_Default)
        
      VectorSourceColor(RGBA(0, 0, 0, 255))
        
      StrokePath(LineWidth * GraphicScale.f, #PB_Path_RoundCorner)

    Else
      
      ;Print Routine No Scaling but printer hard margins
      Startx = Startx - PrinterInfo\LeftMargin
      Starty = Starty - PrinterInfo\TopMargin
      Endx = Endx - PrinterInfo\LeftMargin
      Endy = Endy - PrinterInfo\TopMargin  
      MovePathCursor(Startx, Starty)
        
      AddPathLine(Endx, Endy, #PB_Path_Default)
        
      VectorSourceColor(RGBA(0, 0, 0, 255))
        
      StrokePath(LineWidth,#PB_Path_RoundCorner)

    EndIf

  EndProcedure
 
  Procedure PrintBox(Topx,Topy,Bottomx,Bottomy,LineWidth)
    
    If Preview.i = 1
      
      ;Scaled To Fit Preview Image

      AddPathBox(Topx * GraphicScale.f, Topy * GraphicScale.f, (Bottomx - Topx) * GraphicScale.f, (Bottomy - Topy) * GraphicScale.f)

      VectorSourceColor(RGBA(255, 0, 0, 255))
    
      StrokePath(LineWidth * GraphicScale.f)
 
    Else
      
      ;Print Routine. No Scaling
      Topx = Topx - PrinterInfo\LeftMargin
      Topy = Topy - PrinterInfo\TopMargin
      Bottomx = Bottomx - PrinterInfo\LeftMargin
      Bottomy = Bottomy - PrinterInfo\TopMargin     
      AddPathBox(Topx, Topy , (Bottomx - Topx), (Bottomy - Topy))

      VectorSourceColor(RGBA(255, 0, 0, 255))
            
      StrokePath(LineWidth)
  
  EndIf 
    
  EndProcedure
  
  Procedure PrintText(Startx,Starty,Font.s,Size.i,Text.s)
    
    ;Scaled To Fit Preview Image
    
    If Preview.i = 1
      
      LoadFont(0, Font.s , Size.i )
      
      ASize.f = Size.i * 0.352777778 ;Convert Font Points To mm
      
      MovePathCursor(Startx * GraphicScale.f, Starty * GraphicScale.f)

      VectorFont(FontID(0), ASize.f * GraphicScale.f)
 
      ;VectorSourceColor(RGBA(0, 0, 0, 255))

      DrawVectorText(Text.s)
      
    Else
      
      ;Print Routine. No Scaling
      Startx = Startx - PrinterInfo\LeftMargin
      Starty = Starty - PrinterInfo\TopMargin
      LoadFont(0, Font.s , Size.i )
      ASize.f = Size.i * 0.352777778 ;Convert Font Points To mm
      VectorFont(FontID(0), ASize.f)
      ;VectorSourceColor(RGBA(0, 0, 0, 255))

      MovePathCursor(Startx, Starty)
      DrawVectorText(Text.s)
      
    EndIf

  EndProcedure

  Procedure PrintImage(Image.l,Topx.i,Topy.i,Width.i,Height.i)
    
    If Preview.i = 1

        MovePathCursor(Topx * GraphicScale.f, Topy * GraphicScale.f)
 
        DrawVectorImage(ImageID(Image.l),100,Width.i * GraphicScale.f,Height.i * GraphicScale.f)
 
    Else
            
      ;Print Routine
      Topx = Topx - PrinterInfo\LeftMargin
      Topy = Topy - PrinterInfo\TopMargin
      MovePathCursor(Topx, Topy)
      
      DrawVectorImage(ImageID(Image),255,Width,Height)
 
  EndIf
  
  EndProcedure

  Procedure Finished()

    If Preview.i = 0
      StopVectorDrawing()
      StopPrinting()
    Else
      StopVectorDrawing()
    EndIf
        
  EndProcedure
  
  Procedure DrawPageImage()
    
    ;Calculte Scaling
    If PageHeight > PageWidth
      
      GraphicScale.f = 190/PageHeight
      
    Else
      
      GraphicScale.f = 190/PageWidth
 
    EndIf
    
    ;Create Page Image
    PageImage = CreateImage(#PB_Any,PageWidth.i * GraphicScale.f ,PageHeight.i* GraphicScale.f , 32,RGB(255,255,255))
    
    ;Draw the page image
    If StartDrawing(ImageOutput(PageImage))
         
      ;Add Margin Lines
      FrontColor(RGB(255,0,0)) ; Red lines For Margins
      
      ;Top Margin
      x = Val(GetGadgetText(str_Top))
      LineXY(0,x*GraphicScale.f,190,x * GraphicScale.f) 
       
      ;Left Margin
      x = Val(GetGadgetText(str_Left))
      LineXY(x*GraphicScale.f,0,x * GraphicScale.f, 190)
      
      ;Bottom Margin 
      x = Val(GetGadgetText(str_Bottom))
      LineXY(0,(PageHeight.i-x)*GraphicScale.f,190,(PageHeight.i-x) * GraphicScale.f)
      
      ;Right Margin 
      x = Val(GetGadgetText(str_Right))
      LineXY((PageWidth.i-x)*GraphicScale.f,0,(PageWidth.i-x) * GraphicScale.f, 190) 
        
      StopDrawing()
        
    EndIf
    
    ;Show Page Image
    SetGadgetState(img_Page,ImageID(PageImage))
    
    ;Centre Page Image 
    If PageHeight.i > PageWidth.i
      
      left.i = (190 -GadgetWidth(img_Page))/2
      
    Else
      
     top.i = (190 -GadgetHeight(img_Page))/2
 
    EndIf
    ResizeGadget(img_Page, left.i + 330, top.i + 10, #PB_Ignore, #PB_Ignore) 
    
  EndProcedure
    
  Procedure Page_Setup()
 
  ;Open The Page Setup Window
  wnd_PageSetup = OpenWindow(#PB_Any, x, y, 540, 260, "", #PB_Window_TitleBar | #PB_Window_Tool | #PB_Window_ScreenCentered)
  FrameGadget(#PB_Any, 10, 10, 200, 100, " Select Printer/Paper ")
  btn_Cancel = ButtonGadget(#PB_Any, 430, 220, 100, 30, "Cancel")
  btn_Ok = ButtonGadget(#PB_Any, 320, 220, 100, 30, "Ok")
  cmb_Printers = ComboBoxGadget(#PB_Any, 20, 40, 180, 20)
  DisableGadget(cmb_Printers, 1) ;Enable after loading available printers
  cmb_PaperSize = ComboBoxGadget(#PB_Any, 20, 70, 180, 20)
  FrameGadget(#PB_Any, 10, 110, 310, 100, " Margins")
  TextGadget(#PB_Any, 130, 140, 30, 20, "mm")
  TextGadget(#PB_Any, 130, 170, 30, 20, "mm")
  TextGadget(#PB_Any, 170, 140, 50, 20, "Left", #PB_Text_Right)
  TextGadget(#PB_Any, 20, 140, 50, 20, "Top", #PB_Text_Right)
  TextGadget(#PB_Any, 280, 140, 30, 20, "mm")
  TextGadget(#PB_Any, 280, 170, 30, 20, "mm")
  TextGadget(#PB_Any, 20, 170, 50, 20, "Bottom", #PB_Text_Right)
  TextGadget(#PB_Any, 170, 170, 50, 20, "Right", #PB_Text_Right)
  str_Top = StringGadget(#PB_Any, 80, 140, 40, 20, "")
  str_Bottom = StringGadget(#PB_Any, 80, 170, 40, 20, "")
  str_Left = StringGadget(#PB_Any, 230, 140, 40, 20, "")
  str_Right = StringGadget(#PB_Any, 230, 170, 40, 20, "")
  img_Back = ImageGadget(#PB_Any, 330, 10, 190, 190, 0, #PB_Image_Border)
  img_Page = ImageGadget(#PB_Any, 390, 90, 80, 60, 0)
  opt_Portrait = OptionGadget(#PB_Any, 230, 50, 80, 20, "Portrait")
  opt_Landscape = OptionGadget(#PB_Any, 230, 80, 80, 20, "Landscape")
  FrameGadget(#PB_Any, 220, 10, 100, 100, " Orientation ")
  
  PageBImage = CreateImage(#PB_Any,190 ,190 , 32,RGB(0,0,0))
  SetGadgetState(img_Back,ImageID(PageBImage)) 
  
  App::EnumeratePrinters()
  ForEach App::Printers()       ; Process all the elements...
    AddGadgetItem(cmb_Printers,-1,App::Printers())
  Next
  DisableGadget(cmb_Printers, 0)
  
  ;Test List Only Set On Printer Selection Not yet Available
  AddGadgetItem(cmb_PaperSize,0,"A4")
  AddGadgetItem(cmb_PaperSize,0,"Letter")
  SetGadgetText(cmb_PaperSize,"A4")
   
  ;Default To A4
  PageWidth.i = 210
  PageHeight.i = 297

  TopMargin = 15
  LeftMargin = 15
  BottomMargin = 15
  RightMargin = 15
  Orientation = 1
  SetGadgetState(opt_Portrait,1)
  SetGadgetState(opt_Landscape,0)
  If PageHeight.i < PageWidth.i
     
    temp.i = PageWidth.i
       
    PageWidth.i = PageHeight.i
         
    PageHeight.i = temp.i
       
  EndIf
   
  SetGadgetText(str_Top,Str(TopMargin.i))
  SetGadgetText(str_Left,Str(LeftMargin.i))
  SetGadgetText(str_Bottom,Str(BottomMargin.i))
  SetGadgetText(str_Right,Str(RightMargin.i))
  DrawPageImage()
  
  StickyWindow(wnd_PageSetup,#True)
  OkPressed = #False
  
  Repeat
    
    Event = WaitWindowEvent()
    
    Select event
    
       Case #PB_Event_Menu
         
         Select EventMenu()
             
         EndSelect
         
       Case #PB_Event_Gadget
         
         Select EventGadget()
                       
           Case btn_Ok

             Quit = #True
             OkPressed = #True
             CloseWindow(wnd_PageSetup)
               
           Case btn_Cancel
             Quit = #True
             OkPressed = #False 
             
           Case str_Top
             TopMargin.i = Val(GetGadgetText(str_Top))
             DrawPageImage()  
            
           Case str_Left
             LeftMargin.i = Val(GetGadgetText(str_Left))          
             DrawPageImage()
             
           Case str_Bottom
            BottomMargin.i = Val(GetGadgetText(str_Bottom))          
            DrawPageImage()
             
           Case str_Right
             RightMargin.i = Val(GetGadgetText(str_Right))             
             DrawPageImage() 
             
           Case cmb_Printers
                       
             ;Printer selection
             ;Load List Of Supported Page Sizes And Place In Pages ComboBox
             ;Not Yet Available
             
           Case cmb_PaperSize
                       
             ;Get selected page size in mm
             Select GetGadgetText(cmb_PaperSize)
                 Case "A4"
                   PageWidth.i = 210
                   PageHeight.i= 297
                 Case "Letter"
                   PageWidth.i = 216
                   PageHeight.i= 279
                   
             EndSelect  
             DrawPageImage()
             
           Case opt_Portrait
             
             If GetGadgetState(opt_Portrait) = 1
              ; Orientation.i = 1
               If PageHeight.i < PageWidth.i
                 temp.i = PageWidth.i
                 
                 PageWidth.i = PageHeight.i
                 
                 PageHeight.i = temp.i
                 
               EndIf
             EndIf
           
             ;Redraw Page Image
             DrawPageImage()

           Case opt_Landscape
             If GetGadgetState(opt_Landscape) = 1
             ;  Orientation.i = 0
             
               If PageWidth.i < PageHeight.i 
                 temp.i = PageWidth.i
                 PageWidth.i =PageHeight.i
                PageHeight.i = temp.i
               EndIf
             EndIf
             
             ;Redraw Page Image
             DrawPageImage()
                       
         EndSelect
         
     EndSelect
     
   Until Quit = #True
   
  EndProcedure
  
  Procedure CDPrintEvents(Event)
    
    If event = #PB_Event_CloseWindow
      CloseWindow(PreviewWindow.l)
    EndIf
      
    If Event = #PB_Event_Gadget

      Select EventGadget()
        Case 66
          
          SetGadgetText(66, Str(GetGadgetState(66)))
          If GetGadgetState(66) > 0 And GetGadgetState(66) -1 <= PageNo.i
            ShowPage(GetGadgetState(66) -1)
          ElseIf GetGadgetState(66) < 1
            ;Show Last Page No or first page Number in gadget
            SetGadgetState(66,1)
          Else
            SetGadgetState(66,PageNo.i + 1 )
          EndIf
          
        EndSelect       
         
        EndIf
    
  EndProcedure

EndModule
; IDE Options = PureBasic 5.40 LTS (Windows - x86)
; CursorPosition = 515
; FirstLine = 159
; Folding = DA1
; EnableUnicode
; EnableXP