﻿;{ ==Code Header Comment==============================
;        Name/title: frmMain.pb
;   Executable name: NA Paet development of Report Designer Module
;           Version: 1.0
;            Author: Collectordave
;     Collaborators: Amílcar Matos Pérez
;    Translation by: 
;       Create date: 24\01\2016
; Previous releases: 
; This Release Date: 
;  Operating system: Windows  [X]GUI
;  Compiler version: PureBasic 5.41 (x64),PureBasic 5.4LTS(X86)
;         Copyright: (C)2016
;           License: 
;         Libraries: 
;     English Forum: 
;      French Forum: 
;      German Forum: 
;  Tested platforms: Windows
;       Description: Current report definition file with New() Load() and Save() procedures
; ====================================================
;.......10........20........30........40........50........60........70........80
;}

DeclareModule Report
  
  ;General Report Information variables
  Global Title.s           ;Report Title
  Global DBName.s          ;Database FileName
  Global DBQuery.s         ;Report Query  Global PageHeight.i      ;Height of Selected Page in mm
  Global PageWidth.i       ;Width of Selected Page in mm
  Global PageHeight.i
  Global TopMargin.i       ;Top Margin In mm
  Global LeftMargin.i      ;Left Margin In mm
  Global BottomMargin.i    ;Bottom Margin In mm
  Global RightMargin.i     ;Right Margin In mm
  Global Orientation.i = 1 ;Portrait or Landscape
  Global SectionHeaderFontID.i
  Global SectionFontName.s  ;Display\Print font information
  Global SectionFontSize.i
  Global SectionFontStyle.i
  Global SectionColour.i
  Global FieldFontID.i
  Global FieldFontName.s  ;Display\Print font information
  Global FieldFontSize.i
  Global FieldFontStyle.i
  Global FieldColour.i
  Global PrintPreview.i
  
  Structure HeaderObj
    Type.s      ;Type Of Object to draw
    ID.i        ;Main Object ID
    Text.s      ;The actual text to be shown
    x.i         ;Position and size information
    y.i
    Width.i     ;Actual Image Width
    Height.i    ;Actual Image Height
    MaxWidth.i  ;Maximum Display Width for image
    MaxHeight.i ;Maximum Display Height for image
    FontName.s  ;Display\Print font information
    FontSize.i
    FontStyle.i
    Colour.i
  EndStructure
  
  Structure FieldObj
    Include.i
    FieldID.i
    FieldName.s
    TextID.i
    Text.s
    x.i
    Width.i
  EndStructure
  
  Global NewList NewHeaderObject.HeaderObj()
  
  Global Temp.HeaderObj
  
  Global Dim FieldObject.FieldObj(0)
  
  Declare New()                         ;Procedure to load default report values
  Declare Load(ReportFileName.s)        ;Procedure to load a report definition
  Declare Save()                        ;Procedure to save the report definition
  Declare PrintReport()                 ;Print the loaded report
  Declare RedoImage()
  
EndDeclareModule

Module Report
  
  Global ReportID.i
  Global CurrentYPosition.i = 0
  
Procedure.i CreateReportFile(ReportName.s)
  
  ;Procedure to create an empty report definition file.
  ;returns #True on success
  
  Define Criteria.s
  Define TempResult.i

  TempResult = CreateFile(#PB_Any, ReportName)
  
  If TempResult <> 0
    CloseFile(TempResult)
    ReportID = OpenDatabase(#PB_Any, ReportName, "", "")
    
    ;Create the general Options Table
    Criteria = "CREATE TABLE GlobalDescription (Title VARCHAR(75)  NULL,DBName VARCHAR(250)  NULL,"
    Criteria = Criteria + "DBQuery VARCHAR(250)  NULL,PageWidth INTEGER  NULL,PageHeight INTEGER  NULL,"
    Criteria = Criteria + "TopMargin INTEGER  NULL,LeftMargin INTEGER  NULL,BottomMargin INTEGER  NULL,RightMargin INTEGER  NULL,"
    Criteria = Criteria + "SecColour INTEGER  NULL,SecFontName VARCHAR(25)  NULL,SecFontSize INTEGER  NULL,SecFontStyle INTEGER  NULL,"
    Criteria = Criteria + "FldColour INTEGER  NULL,FldFontName VARCHAR(25)  NULL,FldFontSize INTEGER  NULL,FldFontStyle INTEGER  NULL);"
    DatabaseUpdate(ReportID, Criteria)   
    
    ;Create the HeaderObjects Table
    Criteria = "CREATE TABLE HeaderObjects (Section VARCHAR(20)  NULL,Type VARCHAR(10)  NULL,Text VARCHAR(75)  NULL,"
    Criteria = Criteria + "Top INTEGER  NULL,Left INTEGER  NULL,FontName VARCHAR(30)  NULL,"
    Criteria = Criteria + "FontSize INTEGER  NULL,FontStyle INTEGER  NULL,Colour INTEGER  NULL,Width INTEGER  NULL,Height INTEGER  NULL,MaxWidth INTEGER  NULL,MaxHeight INTEGER  NULL);"
    DatabaseUpdate(ReportID, Criteria)
    
    ;Create the FieldOblects Table    
    Criteria = "CREATE TABLE FieldObjects (Name VARCHAR(20)  NULL,Include INTEGER  NULL,Title VARCHAR(25)  NULL,Start INTEGER  NULL,Width INTEGER  NULL);"
    DatabaseUpdate(ReportID, Criteria)
    
    ProcedureReturn #True
    
  Else
    
    ProcedureReturn #False
    
  EndIf
  
EndProcedure

Procedure New()
  
  ;General Report Information variables
  Title = "New Report"
  DBQuery = ""
  PageWidth = 210
  PageHeight = 287
  TopMargin = 10
  LeftMargin = 10
  BottomMargin = 10
  RightMargin = 10
  Orientation = 1 
  SectionFontName = "Arial"
  SectionFontSize = 14
  SectionFontStyle = 0
  ;Load Default Section Header Font
  SectionHeaderFontID = LoadFont(#PB_Any, SectionFontName, SectionFontSize)
  SectionColour = 255
  FieldFontName = "Arial"
  FieldFontSize = 10
  FieldFontStyle = 0
  ;Load Default Detail section font
  FieldFontID =  LoadFont(#PB_Any, FieldFontName, FieldFontSize)
  FieldColour = 0
  
  ClearList(NewHeaderObject())
  
EndProcedure

Procedure RedoImage()
  
  Define RWidth.f,RHeight.f
  ;Load Image
  If FileSize(NewHeaderObject()\Text) > -1
  
    NewHeaderObject()\ID = LoadImage(#PB_Any, NewHeaderObject()\Text)
    ;Resize image to maintain aspect ratio to maximum sizes set by user
    RWidth = Report::NewHeaderObject()\MaxWidth 
    RHeight = Report::NewHeaderObject()\MaxHeight 
    
    If ImageWidth(Report::NewHeaderObject()\ID) > ImageHeight(Report::NewHeaderObject()\ID)
      RHeight = (RWidth/ ImageWidth(Report::NewHeaderObject()\ID)) * ImageHeight(Report::NewHeaderObject()\ID)
    Else 
      Rwidth = (RHeight/ ImageHeight(Report::NewHeaderObject()\ID)) * ImageWidth(Report::NewHeaderObject()\ID)
    EndIf
    NewHeaderObject()\Height = RHeight
    NewHeaderObject()\Width = Rwidth
    ResizeImage(NewHeaderObject()\ID, Rwidth * App::PPmm, RHeight * App::PPmm)
   
  Else
    NewHeaderObject()\ID = 0
  EndIf
            
  If NewHeaderObject()\ID = 0 ;Failed to load image
    NewHeaderObject()\ID = CreateImage(#PB_Any,NewHeaderObject()\Width * App::PPmm,NewHeaderObject()\Height * App::PPmm,24,RGB($00,$00,$FF))
  EndIf
  
EndProcedure

Procedure LoadImagesAndFonts()
  
  Define Rwidth.f,RHeight.f

  ForEach NewHeaderObject()
    
    Select NewHeaderObject()\Type
        
        Case "Text"
          
          NewHeaderObject()\ID = LoadFont(#PB_Any, NewHeaderObject()\FontName, NewHeaderObject()\FontSize)
          
          If NewHeaderObject()\ID = 0  ;No Font Loaded
            
            ;Use default font
            NewHeaderObject()\ID = GetGadgetFont(#PB_Default)
            
          EndIf
          
        Case "Image"
          
          RedoImage()
                      
    EndSelect

  Next

EndProcedure

Procedure Load(ReportFileName.s)
  
  ;Procedure to load report definition
  Define ObjectCount.i = 1
  
  ;Clear The Object List
  ClearList(NewHeaderObject()) 
    
  If ReportFileName > ""
   
    ;Open report
    ReportID = OpenDatabase(#PB_Any,ReportFileName, "", "") 
    
    ;Fetch General Report Options
    DatabaseQuery(ReportID, "SELECT * FROM GlobalDescription;")
    FirstDatabaseRow(ReportID)
    Report::Title = GetDatabaseString(ReportID,DatabaseColumnIndex(ReportID, "Title")) 
    Report::DBName = GetDatabaseString(ReportID,DatabaseColumnIndex(ReportID, "DBName"))     
    Report::DBQuery = GetDatabaseString(ReportID,DatabaseColumnIndex(ReportID, "DBQuery"))    
    Report::PageWidth = GetDatabaseLong(ReportID,DatabaseColumnIndex(ReportID, "PageWidth"))    
    Report::PageHeight = GetDatabaseLong(ReportID,DatabaseColumnIndex(ReportID, "PageHeight"))     
    Report::TopMargin = GetDatabaseLong(ReportID,DatabaseColumnIndex(ReportID, "TopMargin"))   
    Report::LeftMargin = GetDatabaseLong(ReportID,DatabaseColumnIndex(ReportID, "LeftMargin"))   
    Report::BottomMargin = GetDatabaseLong(ReportID,DatabaseColumnIndex(ReportID, "BottomMargin"))   
    Report::RightMargin = GetDatabaseLong(ReportID,DatabaseColumnIndex(ReportID, "RightMargin"))   
    Report::SectionFontName = GetDatabaseString(ReportID,DatabaseColumnIndex(ReportID, "SecFontName")) 
    Report::SectionFontSize = GetDatabaseLong(ReportID,DatabaseColumnIndex(ReportID, "SecFontSize"))
    Report::SectionFontStyle = GetDatabaseLong(ReportID,DatabaseColumnIndex(ReportID, "SecFontStyle"))
    ;Load Report Section Header Font
    Report::SectionHeaderFontID = LoadFont(#PB_Any, SectionFontName, SectionFontSize)
    Report::SectionColour = GetDatabaseLong(ReportID,DatabaseColumnIndex(ReportID, "SecColour"))
    Report::FieldFontName = GetDatabaseString(ReportID,DatabaseColumnIndex(ReportID, "FldFontName")) 
    Report::FieldFontSize = GetDatabaseLong(ReportID,DatabaseColumnIndex(ReportID, "FldFontSize"))
    Report::FieldFontStyle = GetDatabaseLong(ReportID,DatabaseColumnIndex(ReportID, "FldFontStyle"))
    ;Load Report Detail Section Font
    Report::FieldFontID =  LoadFont(#PB_Any, FieldFontName, FieldFontSize)
    Report::FieldColour = GetDatabaseLong(ReportID,DatabaseColumnIndex(ReportID, "FldColour"))
     
    FinishDatabaseQuery(ReportID)
    
    ;Fetch Report Header Objects
    DatabaseQuery(ReportID, "SELECT * FROM HeaderObjects;")
    FirstDatabaseRow(ReportID)
    
    AddElement(NewHeaderObject())
    NewHeaderObject()\Type   = GetDatabaseString(ReportID,DatabaseColumnIndex(ReportID, "Type")) 
    NewHeaderObject()\Text   = GetDatabaseString(ReportID,DatabaseColumnIndex(ReportID, "Text"))    
    NewHeaderObject()\x = GetDatabaseLong(ReportID,DatabaseColumnIndex(ReportID, "Left"))     
    NewHeaderObject()\y = GetDatabaseLong(ReportID,DatabaseColumnIndex(ReportID, "Top")) 
    NewHeaderObject()\FontName = GetDatabaseString(ReportID,DatabaseColumnIndex(ReportID, "FontName"))     
    NewHeaderObject()\FontSize = GetDatabaseLong(ReportID,DatabaseColumnIndex(ReportID, "FontSize")) 
    NewHeaderObject()\FontStyle = GetDatabaseLong(ReportID,DatabaseColumnIndex(ReportID, "FontStyle")) 
    NewHeaderObject()\Colour = GetDatabaseLong(ReportID,DatabaseColumnIndex(ReportID, "Colour")) 
    NewHeaderObject()\Width = GetDatabaseLong(ReportID,DatabaseColumnIndex(ReportID, "Width")) 
    NewHeaderObject()\Height = GetDatabaseLong(ReportID,DatabaseColumnIndex(ReportID, "Height")) 
    NewHeaderObject()\MaxWidth = GetDatabaseLong(ReportID,DatabaseColumnIndex(ReportID, "MaxWidth")) 
    NewHeaderObject()\MaxHeight = GetDatabaseLong(ReportID,DatabaseColumnIndex(ReportID, "MaxHeight"))     

    While NextDatabaseRow(ReportID)
      
      AddElement(NewHeaderObject())
      NewHeaderObject()\Type   = GetDatabaseString(ReportID,DatabaseColumnIndex(ReportID, "Type")) 
      NewHeaderObject()\Text   = GetDatabaseString(ReportID,DatabaseColumnIndex(ReportID, "Text"))    
      NewHeaderObject()\x = GetDatabaseLong(ReportID,DatabaseColumnIndex(ReportID, "Left"))     
      NewHeaderObject()\y = GetDatabaseLong(ReportID,DatabaseColumnIndex(ReportID, "Top")) 
      NewHeaderObject()\FontName = GetDatabaseString(ReportID,DatabaseColumnIndex(ReportID, "FontName"))     
      NewHeaderObject()\FontSize = GetDatabaseLong(ReportID,DatabaseColumnIndex(ReportID, "FontSize")) 
      NewHeaderObject()\FontStyle = GetDatabaseLong(ReportID,DatabaseColumnIndex(ReportID, "FontStyle")) 
      NewHeaderObject()\Colour = GetDatabaseLong(ReportID,DatabaseColumnIndex(ReportID, "Colour")) 
      NewHeaderObject()\Width = GetDatabaseLong(ReportID,DatabaseColumnIndex(ReportID, "Width")) 
      NewHeaderObject()\Height = GetDatabaseLong(ReportID,DatabaseColumnIndex(ReportID, "Height"))      
      NewHeaderObject()\MaxWidth = GetDatabaseLong(ReportID,DatabaseColumnIndex(ReportID, "MaxWidth")) 
      NewHeaderObject()\MaxHeight = GetDatabaseLong(ReportID,DatabaseColumnIndex(ReportID, "MaxHeight"))     
 
    Wend
    FinishDatabaseQuery(ReportID)
    LoadImagesAndFonts()
    
    ;Fetch Report Field Objects
    ObjectCount = 1
    DatabaseQuery(ReportID, "SELECT * FROM FieldObjects;")
    FirstDatabaseRow(ReportID)
    ReDim FieldObject(ObjectCount)
    FieldObject(0)\FieldName = GetDatabaseString(ReportID,DatabaseColumnIndex(ReportID, "Name"))
    FieldObject(0)\Include = GetDatabaseLong(ReportID,DatabaseColumnIndex(ReportID, "Include"))
    FieldObject(0)\Text = GetDatabaseString(ReportID,DatabaseColumnIndex(ReportID, "Title"))
    FieldObject(0)\x = GetDatabaseLong(ReportID,DatabaseColumnIndex(ReportID, "Start"))   
    FieldObject(0)\Width = GetDatabaseLong(ReportID,DatabaseColumnIndex(ReportID, "Width"))      
    While NextDatabaseRow(ReportID)
      ObjectCount = ObjectCount + 1
      ReDim FieldObject(ObjectCount)
      FieldObject(ObjectCount-1)\FieldName = GetDatabaseString(ReportID,DatabaseColumnIndex(ReportID, "Name"))
      FieldObject(ObjectCount-1)\Include = GetDatabaseLong(ReportID,DatabaseColumnIndex(ReportID, "Include"))     
      FieldObject(ObjectCount-1)\Text = GetDatabaseString(ReportID,DatabaseColumnIndex(ReportID, "Title"))
      FieldObject(ObjectCount-1)\x = GetDatabaseLong(ReportID,DatabaseColumnIndex(ReportID, "Start"))   
      FieldObject(ObjectCount-1)\Width = GetDatabaseLong(ReportID,DatabaseColumnIndex(ReportID, "Width"))       
    Wend
    FinishDatabaseQuery(ReportID)    
    
    ;Close The Report Database
    CloseDatabase(ReportID)
    
    ProcedureReturn #True
    
    Else
      ProcedureReturn #False
  EndIf
    
EndProcedure 

Procedure Save()
  
  ;Procedure to Save  Report definition
  Define Pattern.s
  Define ObjectCount.i = 0
  Define ReportFileName.s
  Define Criteria.s
  
  Pattern = "Reports (*.rpt)|*.rpt;|All files (*.*)|*.*"
  ReportFileName = SaveFileRequester("Please choose file to save", GetCurrentDirectory(), Pattern, 0)
  
  ;Check FileName extension part
  If FindString(ReportFileName,".") > 0
    ReportFileName = Left(ReportFileName,FindString(ReportFileName,".") -1 ) + ".rpt"
  Else
    ReportFileName = ReportFileName + ".rpt"
  EndIf
  
  If App::FileExists(ReportFileName)
     If  App::Message("File Exists", "Would You Like To Overwrite?",App::#YesNo|App::#WarningIcon)  = App:: #MsgNo
       ProcedureReturn      ;Quit Save Operation   
    EndIf 
  EndIf
  
  If CreateReportFile(ReportFileName) = #True

    ;Save General Settings
    Criteria = "INSERT INTO GlobalDescription"
    Criteria = Criteria + "(Title,DBName,DBQuery,PageWidth,PageHeight,TopMargin,LeftMargin,BottomMargin,RightMargin,SecColour,SecFontName,SecFontSize,SecFontStyle,FldColour,FldFontName,FldFontSize,FldFontStyle)"
    Criteria = Criteria + " VALUES('" + Report::Title + "','" + Report::DBName + "','" + Report::DBQuery + "',"
    Criteria = Criteria + Str(Report::PageWidth) + "," + Str(Report::PageHeight) + "," + Str(Report::TopMargin) + "," 
    Criteria = Criteria + Str(Report::LeftMargin) + "," + Str(Report::BottomMargin) + "," + Str(Report::RightMargin) + ","
    Criteria = Criteria + Str(Report::SectionColour) + ",'" + Report::SectionFontName + "'," + Str(Report::SectionFontSize) +"," + Str(Report::SectionFontStyle) + ","
    Criteria = Criteria + Str(Report::FieldColour) + ",'" + Report::FieldFontName + "'," + Str(Report::FieldFontSize) +"," + Str(Report::FieldFontStyle) + ");"
    
    DatabaseUpdate(ReportID, Criteria)
    
    ;Save Header Objects
    ForEach Report::NewHeaderObject()
      Criteria = "INSERT INTO HeaderObjects"
      Criteria = Criteria + "(Type,Text,Left,Top,FontName,FontSize,FontStyle,Colour,Width,Height,MaxWidth,MaxHeight)"
      Criteria = Criteria + " VALUES('" + NewHeaderObject()\Type + "','" + NewHeaderObject()\Text + "',"
      Criteria = Criteria + Str(NewHeaderObject()\x) + "," + Str(NewHeaderObject()\y) + ",'" + NewHeaderObject()\FontName + "'," 
      Criteria = Criteria + Str(NewHeaderObject()\FontSize) + "," + Str(NewHeaderObject()\FontStyle) + "," + Str(NewHeaderObject()\Colour) + ","
      Criteria = Criteria + Str(NewHeaderObject()\Width) + "," +  Str(NewHeaderObject()\Height) + "," + Str(NewHeaderObject()\MaxWidth) + "," +  Str(NewHeaderObject()\MaxHeight)+ ");"
      DatabaseUpdate(ReportID, Criteria)   
    Next
    
    ;Save Field Objects
    For ObjectCount = 0 To ArraySize(FieldObject()) -1
      Criteria = "INSERT INTO FieldObjects"
      Criteria = Criteria + "(Name,Include,Title,Start,Width)"
      Criteria = Criteria + " VALUES('" + Report::FieldObject(ObjectCount)\FieldName + "'," + Str(Report::FieldObject(ObjectCount)\Include) + ",'"
      Criteria = Criteria + Report::FieldObject(ObjectCount)\Text + "  '," + Str(Report::FieldObject(ObjectCount)\x) + "," + Str(Report::FieldObject(ObjectCount)\Width) + ");"
      DatabaseUpdate(ReportID, Criteria)
    Next ObjectCount
   
    CloseDatabase(ReportID)
  
  Else
    App::Message("Error", "Failed to create report file",App::#OkOnly|App::#WarningIcon)  
  EndIf

EndProcedure

Procedure PrintReportHeader()
  
  ForEach NewHeaderObject()
    
    Select NewHeaderObject()\Type
        
      Case "Text"

        VectorSourceColor(RGBA(Red(NewHeaderObject()\Colour),Green(NewHeaderObject()\Colour),Blue(NewHeaderObject()\Colour),255))
        CDPrint::PrintText(NewHeaderObject()\x + LeftMargin,NewHeaderObject()\y,NewHeaderObject()\FontName,NewHeaderObject()\FontSize,NewHeaderObject()\Text)
        If NewHeaderObject()\y > CurrentYPosition
          CurrentYPosition = NewHeaderObject()\y
        EndIf
        
           
      Case "Image"

        CDPrint::PrintImage(NewHeaderObject()\ID,NewHeaderObject()\x + LeftMargin,NewHeaderObject()\y,NewHeaderObject()\Width,NewHeaderObject()\Height)  
        If NewHeaderObject()\y + NewHeaderObject()\Height > CurrentYPosition
          CurrentYPosition = NewHeaderObject()\y + NewHeaderObject()\Height
        EndIf
        
    EndSelect
  
  Next 
  CurrentYPosition = CurrentYPosition + 5
  
EndProcedure

Procedure PrintSectionHeader()
  
  ;Print Section Header
  VectorSourceColor(RGBA(Red(SectionColour),Green(SectionColour),Blue(SectionColour),255))
  For iLoop = 0 To ArraySize(FieldObject()) -1
    If FieldObject(iLoop)\Include = #True
      CDPrint::PrintText(FieldObject(iLoop)\x + LeftMargin,CurrentYPosition,SectionFontName,SectionFontSize,FieldObject(iLoop)\Text)
    EndIf  
  Next iLoop
  
EndProcedure

Procedure PrintReport()
  
  ;Procedure to print a previously loaded report
  
  ;Check If Report Loaded
  If DBQuery = ""
    App::Message("Error", "No Report Loaded Aborting Print\Preview",App::#OkOnly|App::#WarningIcon)     
    ProcedureReturn
  EndIf
  
  Define Page.i = 0 ;Holds current Page number
  Define Printing.i = #True ;True while printing report set to False to stop
  Define SectionHeight.i,DetailHeight.i

  SectionHeight = (SectionFontSize * 0.352777778) + 4 
  DetailHeight = (FieldFontSize * 0.352777778) + 2  
   
  CurrentYPosition = TopMargin  
  
  ;Start The Print Job
  PrintPreview = CDPrint::open(Title,PageHeight,PageWidth)
  
  PrintReportHeader()

  ;CurrentYPosition = 40
  
  PrintSectionHeader() ;First section header after report header
  CurrentYPosition = CurrentYPosition + SectionHeight
  
  ;Open Main Database
  DBHnd.i = OpenDatabase(#PB_Any, DBName.s, #Empty$, #Empty$)  
  If DBHnd.i = 0
     App::Message("Database Error", "Failed to open database!",App::#OkOnly|App::#WarningIcon) 
    ProcedureReturn

  EndIf
  
  ;Run DBQuery
  DatabaseQuery(DBHnd,DBQuery)
  FirstDatabaseRow(DBHnd)
  ;Print first row of data
  VectorSourceColor(RGBA(Red(FieldColour),Green(FieldColour),Blue(FieldColour),255))

  For iLoop = 0 To ArraySize(FieldObject()) -1
    If FieldObject(iLoop)\Include = #True
      PText.s = GetDatabaseString(DBHnd, DatabaseColumnIndex(DBHnd, FieldObject(iLoop)\FieldName))
      CDPrint::PrintText(FieldObject(iLoop)\x + LeftMargin,CurrentYPosition,FieldFontName,FieldFontSize,PText)
    EndIf
  Next iLoop
  
  CurrentYPosition = CurrentYPosition + DetailHeight
  
  While Printing = #True

    While NextDatabaseRow(DBHnd)
      ;Print Row of data
      For iLoop = 0 To ArraySize(FieldObject()) -1
        If FieldObject(iLoop)\Include = #True
          VectorSourceColor(RGBA(Red(FieldColour),Green(FieldColour),Blue(FieldColour),255))
          PText.s = GetDatabaseString(DBHnd, DatabaseColumnIndex(DBHnd, FieldObject(iLoop)\FieldName))
          CDPrint::PrintText(FieldObject(iLoop)\x + LeftMargin,CurrentYPosition,FieldFontName,FieldFontSize,PText)
        EndIf
      Next iLoop
  
      CurrentYPosition = CurrentYPosition + DetailHeight
    
      If CurrentYPosition > 250
        CDPrint::AddPage(PageHeight,PageWidth)
        CurrentYPosition = TopMargin + 1
        PrintSectionHeader() ;First section header after report header
        CurrentYPosition = CurrentYPosition + SectionHeight
      EndIf
    
    Wend
    
    ;No more data so finish printing
    Printing = #False
  Wend
  
  CloseDatabase(DBHnd)
  CDPrint::Finished()
  
EndProcedure

EndModule
; IDE Options = PureBasic 5.40 LTS (Windows - x86)
; CursorPosition = 192
; FirstLine = 119
; Folding = nI-
; EnableUnicode
; EnableXP